const path = require('path');
const webpack = require('webpack');
const SpeedMeasurePlugin = require('speed-measure-webpack-plugin');
const {CheckerPlugin} = require('awesome-typescript-loader');

const smp = new SpeedMeasurePlugin();

const {NODE_ENV} = process.env;

module.exports = smp.wrap({
  mode: NODE_ENV === 'production' ? NODE_ENV : 'development',

  // the place webpack will start when building your bundles
  entry: [path.resolve(__dirname, '../src/web/index')],

  // sets up rules for any special importers
  module: {
    rules: [
      {
        test: /\.tsx?$/,
        use: 'awesome-typescript-loader',
        exclude: /node_modules/,
      },
      {
        test: /\.mjs$/,
        include: /node_modules/,
        type: 'javascript/auto',
      },
    ],
  },

  // sets up plugins
  plugins: [
    new webpack.DefinePlugin({
      __isBrowser__: 'true',
    }),
    new CheckerPlugin(),
  ],

  // Enable sourcemaps for debugging webpack's output.
  devtool: 'cheap-module-eval-source-map',

  // file extensions for webpack to look at
  resolve: {
    // extensions: ['*', '.js', '.jsx', '.mjs', 'ts', 'tsx'],
    extensions: ['.ts', '.tsx', '.js', '.jsx', '.mjs', '.css', '.scss'],

    // alias: {
    //   react: 'preact-compat',
    //   'react-dom': 'preact-compat',
    //   // Not necessary unless you consume a module using `createClass`
    //   'create-react-class': 'preact-compat/lib/create-react-class',
    //   // Not necessary unless you consume a module requiring `react-dom-factories`
    //   'react-dom-factories': 'preact-compat/lib/react-dom-factories',
    // },
  },
  // where webpack will output your finished bundle
  output: {
    path: path.resolve(__dirname, '../dist/web'),
    publicPath: '/',
    filename: 'bundle.js',
  },
});
