import 'isomorphic-fetch';

import dotenv from 'dotenv';
import Koa from 'koa';
import logger from 'koa-logger';
import session from 'koa-session';
import views from 'koa-views';
import serve from 'koa-static';
import createShopifyAuth, {verifyRequest} from '@shopify/koa-shopify-auth';
import koaWebpack from 'koa-webpack';
import graphQLProxy from '@shopify/koa-shopify-graphql-proxy';
import mount from 'koa-mount';
import webpack from 'webpack';
import path from 'path';

import configWebpack from '../../config/webpack.web';
import homeRouter from './router/homeRouter';
import renderReactApp from './render-react-app';

dotenv.config();

const server = () => {
  const {SHOPIFY_SECRET, SHOPIFY_API_KEY} = process.env;

  const compiler = webpack(configWebpack as any);
  const pView = path.join(__dirname, '../public');
  const pView2 = path.join(__dirname, '../public/static');
  const {PORT = 3000} = process.env;

  const app = new Koa();

  app.use(session(app));
  app.use(
    views(pView, {
      extension: 'ejs',
    }),
  );
  if (process.env.NODE_ENV !== 'production') {
    app.use(logger());
  }

  app.keys = [SHOPIFY_SECRET];

  console.log(SHOPIFY_SECRET, SHOPIFY_API_KEY);

  app.use(homeRouter.routes());
  app.use(serve(pView2));

  app.use(
    createShopifyAuth({
      apiKey: SHOPIFY_API_KEY,
      secret: SHOPIFY_SECRET,
      scopes: ['write_products'],
      afterAuth(ctx) {
        const {shop, accessToken} = ctx.session;

        console.log('We did it!', shop, accessToken);

        ctx.redirect('/');
      },
    }),
  );

  app.use(mount('/shopify', graphQLProxy()));

  app.use(
    verifyRequest({
      authRoute: '/auth',
      fallbackRoute: '/install',
    }),
  );

  const config2: koaWebpack.Options = {
    compiler: compiler,
    devMiddleware: {
      publicPath: '/',
      filename: 'bundle.js',
      serverSideRender: true,
    },
  };

  koaWebpack(config2).then((middleware) => {
    app.use(middleware);
    app.use(renderReactApp);
  });

  // app.use(renderReactApp);

  // start the server on the given port
  app.listen(PORT, () => {
    console.log(`🚀 Listening on port ${PORT}`);
  });
};

server();

export default server;
