import React, {Fragment} from 'react';
import Fetch from 'react-fetch-component';
import {Mutation} from 'react-apollo';
import {gql} from 'apollo-boost';

import GameList from '../components/GameList';

const CREATE_PRODUCT = gql`
  mutation CreateProduct($product: ProductInput!) {
    productCreate(input: $product) {
      product {
        id
        title
      }
    }
  }
`;

export default function() {
  return (
    <Fragment>
      <h1>Board game loaders</h1>
      <Fetch url="https://boardgameslist.herokuapp.com" as="json">
        {(fetchResults) => {
          if (fetchResults.loading) {
            return <p>Loading</p>;
          }

          if (fetchResults.error) {
            return <p>fialed to fetch games</p>;
          }

          return (
            <Mutation mutation={CREATE_PRODUCT}>
              {(createProduct, mutationResults) => {
                const loading = mutationResults.loading && <p>loading... </p>;

                const error = mutationResults.error && (
                  <p>error creating product</p>
                );

                const success = mutationResults.data && (
                  <p>
                    successfully created &nbsp;
                    {mutationResults.data.productCreate.product.title}
                  </p>
                );

                return (
                  <Fragment>
                    <GameList
                      games={fetchResults.data}
                      onAddGame={(title, image) => {
                        const productInput = {
                          title,
                          productType: 'board game',
                          images: {
                            src: image || '',
                          },
                        };

                        createProduct({
                          variables: {product: productInput},
                        });
                      }}
                    />
                    {loading}
                    {error}
                    {success}
                  </Fragment>
                );
              }}
            </Mutation>
          );
        }}
      </Fetch>
    </Fragment>
  );
}
